// ChunkTransfert.cs
//
// PRESTO Starter Kit for Microsoft .NET 3.0
// 2007 Microsoft France
// Published under the CECILL-B Free Software license agreement.
// (http://www.cecill.info/licences/Licence_CeCILL-B_V1-en.txt)
// 
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, 
// WHETHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED 
// WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE. 
// THE ENTIRE RISK OF USE OR RESULTS IN CONNECTION WITH THE USE OF THIS CODE 
// AND INFORMATION REMAINS WITH THE USER. 
//

using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.ServiceModel;
using System.Threading;
using System.Runtime.Remoting.Messaging;

namespace Microsoft.Presto.StarterKit.NonNormative
{
    public class ChunkingChannel
    {
        FileSystemWatcher messageWatcher = null;
        private int chunkSize ;

        public ChunkingChannel(int chunkSize)
        {
            DirectoryInfo dinfo = new DirectoryInfo(Constants.Payloads.outgoingFolder);
            if (!dinfo.Exists) dinfo.Create();
            Console.WriteLine("Waiting for message payload on " + dinfo.FullName);

            messageWatcher = new FileSystemWatcher();
            messageWatcher.Filter = "*.*";
            messageWatcher.Path = dinfo.FullName;
            messageWatcher.Created += new FileSystemEventHandler(messageWatcher_Created);
            messageWatcher.EnableRaisingEvents = true;

            this.chunkSize = chunkSize;
        }

        void messageWatcher_Created(object sender, FileSystemEventArgs e)
        {
            Console.WriteLine("   Submitting {0} to remote Endpoint...", e.Name);

            TransfertDelegate del = new TransfertDelegate(SubmitOneWay);
            del.BeginInvoke(e.FullPath, new AsyncCallback(EndSubmitOneWay), this);
        }

        private delegate void TransfertDelegate(string filename); 
        private void SubmitOneWay(string filename)
        {
            ChannelFactory<IChunking> factory = new ChannelFactory<IChunking>("prestoChunking");
            IChunking proxy = factory.CreateChannel();

            FileInfo finfo = new FileInfo(filename);
            Decimal d = Decimal.Divide(finfo.Length, chunkSize);
            int nbrChunk = ((int)Math.Ceiling(d));
            Console.WriteLine("   (chunk size: {0}, number of chunks: {1})", chunkSize, nbrChunk);

            FileStream fstream = null;
            proxy.StartSubmitOneWay(finfo.Name, nbrChunk);

            while (true)
            {
                try
                {
                    fstream = File.OpenRead(finfo.FullName);
                    break;
                }
                catch (Exception ex)
                {
                    // the file has not finished to copy 
                    Thread.Sleep(100);
                }
            }

            try
            {          
                for (int i = 0; i < nbrChunk; i++)
                {
                    Chunk chunk = new Chunk();
                    chunk.id = i;
                    chunk.startPosition = i * chunkSize;
                    fstream.Position = chunk.startPosition;

                    byte[] buffer = new byte[chunkSize];
                    chunk.length = fstream.Read(buffer, 0, chunkSize);
                    chunk.content = buffer;

                    Console.WriteLine("   ...Submitting chunk #{0}", chunk.id);
                    proxy.SendSubmitOneWayChunk(chunk);
                }
            }
            finally
            {
                proxy.EndStartSubmitOneWay();
                fstream.Close();
                finfo.Delete();
            }
            Console.WriteLine("   Done.");
        }

        private void EndSubmitOneWay(IAsyncResult result)
        {
            AsyncResult aResult = (AsyncResult)result;
            TransfertDelegate dd = (TransfertDelegate)aResult.AsyncDelegate;
            dd.EndInvoke(result);
        }
    }
}
