// SourceApplication.cs
//
// PRESTO Starter Kit for Microsoft .NET 3.0
// 2007 Microsoft France
// Published under the CECILL-B Free Software license agreement.
// (http://www.cecill.info/licences/Licence_CeCILL-B_V1-en.txt)
// 
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, 
// WHETHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED 
// WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE. 
// THE ENTIRE RISK OF USE OR RESULTS IN CONNECTION WITH THE USE OF THIS CODE 
// AND INFORMATION REMAINS WITH THE USER. 
//

using System;
using System.Configuration;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.IO;
using System.Xml;
using System.Windows.Forms;

namespace Microsoft.Presto.StarterKit
{
    public partial class formSourceApplication : Form
    {
        private PrestoSoapNodeConfig config = new PrestoSoapNodeConfig();
        private PrestoChannelFactory channelFactory = new PrestoChannelFactory();

        public formSourceApplication()
        {
            InitializeComponent();

            // Service endpoint base addresses
            config.ServiceEndpointUri = ConfigurationManager.AppSettings["baseAddress"];
            config.ServiceEndpointMexUri = ConfigurationManager.AppSettings["baseMexAddress"];
            config.SoapIntermediaryEndpointUri = ConfigurationManager.AppSettings["baseListenToAddress"];

            // Service identiy and credentials (not used with MEX support)
            config.ServiceIdentity = ConfigurationManager.AppSettings["identity"];
            config.ServiceCertificateThumbprint = ConfigurationManager.AppSettings["serviceCertificateThumbprint"];

            // Client credentials
            config.ClientCertificateThumbprint = ConfigurationManager.AppSettings["clientCertificateThumbprint"];

            this.prestoSoapNodeConfigBindingSource.DataSource = config;
            this.channelFactory.Config = config;

            this.textBoxPayloadPath.Text = config.DocumentsDirectory + config.DocumentToSend;
            this.config.UseManualSettings = true;

            OutputTrace.OnLogMessage += new OutputTrace.LogDelegate(OutputTrace_OnLogMessage);
        }

        void OutputTrace_OnLogMessage(string message)
        {
            this.BeginInvoke(new OutputTrace.LogDelegate(LogMessage), new object[] {message});           
        }

        private void LogMessage(string message)
        {
            this.textBoxTrace.AppendText(Environment.NewLine + message);
        }

        private void buttonSendOneWay_Click(object sender, EventArgs e)
        {
            this.channelFactory.SendMessageToIPrestoEndpoint(this.textBoxPayloadPath.Text);
        }

        private void buttonSendResquest4Reply_Click(object sender, EventArgs e)
        {
            this.channelFactory.SendMessageToIPresto2Endpoint(this.textBoxPayloadPath.Text, checkBoxSaveReply.Checked); 
        }

        private void buttonClearTrace_Click(object sender, EventArgs e)
        {
            textBoxTrace.Text = "";
        }

        private void checkBoxUseSoapIntermediary_CheckedChanged(object sender, EventArgs e)
        {
            textBoxSoapIntermediaryEndpointUri.Enabled = checkBoxUseSoapIntermediary.Checked;
        }

        private void EnableManualConfigurationSettings(bool enable)
        {
            comboBoxAddressingVersion.Enabled = enable;
            //checkBoxUseReliableMessaging.Enabled = enable;
            //comboBoxEncodingMode.Enabled = enable;
            checkBoxUseSignature.Enabled = enable;
            checkBoxUseEncryption.Enabled = enable;
        }

        private void radioButtonUseMetadataExchange_CheckedChanged(object sender, EventArgs e)
        {
            EnableManualConfigurationSettings(!radioButtonUseMetadataExchange.Checked);
            checkBoxUseReliableMessaging.Enabled = !radioButtonUseMetadataExchange.Checked;
            comboBoxEncodingMode.Enabled = !radioButtonUseMetadataExchange.Checked;

            if (radioButtonUseMetadataExchange.Checked == true)
            {
                config.UseConfigurationFile = false;
                config.UseManualSettings = false;
            }
        }

        private void radioButtonConfigurationFile_CheckedChanged(object sender, EventArgs e)
        {
            EnableManualConfigurationSettings(!radioButtonConfigurationFile.Checked);
            
            if (radioButtonConfigurationFile.Checked == true)
            {
                config.UseMetadataExchange = false;
                config.UseManualSettings = false;
            }
        }

        private void radioButtonManualConfiguration_CheckedChanged(object sender, EventArgs e)
        {
            EnableManualConfigurationSettings(radioButtonManualConfiguration.Checked);
            
            if (radioButtonManualConfiguration.Checked == true)
            {
                config.UseMetadataExchange = false;
                config.UseConfigurationFile = false;
            }
        }

        private void textBoxPayloadPath_TextChanged(object sender, EventArgs e)
        {
            if (File.Exists(textBoxPayloadPath.Text))
            {
                this.buttonSendResquest4Reply.Enabled = true;
                this.buttonSendOneWay.Enabled = true;
                this.textBoxPayloadPath.ForeColor = Color.Black;
            }
            else
            {
                this.buttonSendResquest4Reply.Enabled = false;
                this.buttonSendOneWay.Enabled = false;
                this.textBoxPayloadPath.ForeColor = Color.Red;
            }
        }

        private void buttonFile_Click(object sender, EventArgs e)
        {
            openFileDialog4Payload.ShowDialog();
            textBoxPayloadPath.Text = openFileDialog4Payload.FileName;
        }

        private void checkBoxAutoCompleteUris_CheckedChanged(object sender, EventArgs e)
        {
            if (checkBoxAutoCompleteUris.Checked == true)
            {
                // Reload initial address settings
                config.ServiceEndpointUri = ConfigurationManager.AppSettings["baseAddress"];
                this.textBoxServiceEndpointUri.Text = config.ServiceEndpointUri; 
                config.SoapIntermediaryEndpointUri = ConfigurationManager.AppSettings["baseListenToAddress"];
                this.textBoxSoapIntermediaryEndpointUri.Text = config.SoapIntermediaryEndpointUri;
            } 
        }

        private void checkBoxUseEncryption_CheckedChanged(object sender, EventArgs e)
        {
            if (checkBoxUseEncryption.Checked)
            {
                checkBoxUseSignature.Checked = true;
                config.UseDigitalSignature = true;

            }
        }


        private void radioButtonNoAuthentication_CheckedChanged(object sender, EventArgs e)
        {
            if (radioButtonNoAuthentication.Checked)
            {
                checkBoxUseSignature.Checked = false;
                checkBoxUseEncryption.Checked = false;
                checkBoxUseSignature.Enabled = false;
                checkBoxUseEncryption.Enabled = false;
            }

        }

        private void radioButtonAnonymousForCertificate_CheckedChanged(object sender, EventArgs e)
        {
            if (radioButtonAnonymousForCertificate.Checked)
            {
                checkBoxUseSignature.Enabled = true;
                checkBoxUseEncryption.Enabled = true;
            }
        }

        private void radioButtonMutualCertificate_CheckedChanged(object sender, EventArgs e)
        {
            if (radioButtonMutualCertificate.Checked)
            {
                checkBoxUseSignature.Enabled = true;
                checkBoxUseEncryption.Enabled = true;
            }
        }

    }
}