// PrestoRibbon.cs
//
// PRESTO Starter Kit for Microsoft .NET 3.0
// 2007 Microsoft France
// Published under the CECILL-B Free Software license agreement.
// (http://www.cecill.info/licences/Licence_CeCILL-B_V1-en.txt)
// 
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, 
// WHETHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED 
// WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE. 
// THE ENTIRE RISK OF USE OR RESULTS IN CONNECTION WITH THE USE OF THIS CODE 
// AND INFORMATION REMAINS WITH THE USER. 
//

using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Drawing;
using System.IO;
using System.Text;
using System.Reflection;
using System.Runtime.InteropServices;
using System.Windows.Forms;
using Office = Microsoft.Office.Core;
using Microsoft.Office.Interop.Word;
using Microsoft.Office.Tools;

namespace WordAddIn4Presto
{
    // This is an override of the RequestService method in the ThisAddIn class.
    // To hook up your custom ribbon uncomment this code.
    public partial class ThisAddIn
    {
        private PrestoRibbon ribbon;

        protected override object RequestService(Guid serviceGuid)
        {
            if (serviceGuid == typeof(Office.IRibbonExtensibility).GUID)
            {
                if (ribbon == null)
                    ribbon = new PrestoRibbon();
                return ribbon;
            }

            return base.RequestService(serviceGuid);
        }
    }

    [ComVisible(true)]
    public class PrestoRibbon : Office.IRibbonExtensibility
    {
        private Office.IRibbonUI ribbon;

        public PrestoRibbon()
        {
        }

        #region IRibbonExtensibility Members

        public string GetCustomUI(string ribbonID)
        {
            //return GetResourceText("WordAddIn4Presto.PrestoRibbon.xml");
            return Properties.Resources.PrestoRibbon;
        }

        #endregion

        #region Ribbon Callbacks

        public void OnLoad(Office.IRibbonUI ribbonUI)
        {
            this.ribbon = ribbonUI;
        }

        private bool isTaskPaneVisible;
        public bool IsTaskPaneVisible
        {
            get { return isTaskPaneVisible; }
            set
            {
                isTaskPaneVisible = value;
                ribbon.InvalidateControl("togglePrestoButton");
            }
        }

        public bool GetPressed(Office.IRibbonControl control)
        {
            switch (control.Id)
            {
                case "togglePrestoButton":
                    return isTaskPaneVisible;
                default:
                    return false;
            }
        }

        public void OntogglePrestoButton(Office.IRibbonControl control, bool isPressed)
        {
            UpdateTaskPane(isPressed, "Send To Presto");

        }
        public void InvalidateControl(string ctpTitle)
        {
            ribbon.InvalidateControl("toggleButton" + ctpTitle);
        }


        /// <summary>
        /// Handling active Document
        /// </summary>
        /// <param name="isPressed"></param>
        /// <param name="title"></param>
        private static void UpdateTaskPane(bool isPressed, string title)
        {
            Document doc = Globals.ThisAddIn.Application.ActiveDocument;
            Window activeWindow = (Window)doc.ActiveWindow;
            Globals.ThisAddIn.RemoveTaskPane(activeWindow);
            CustomTaskPane ctp = Globals.ThisAddIn.AddTaskPane(activeWindow, title);
            ctp.Visible = isPressed;
        }

        public stdole.IPictureDisp GetImage(Office.IRibbonControl control)
        {
            stdole.IPictureDisp image = null;

            switch (control.Id)
            {
                case "togglePrestoButton":
                    image = PictureDispMaker.ConvertImage(Properties.Resources.dgme);
                    break;
            }
            return image;
        }

        #endregion

        #region Helpers

        private static string GetResourceText(string resourceName)
        {
            Assembly asm = Assembly.GetExecutingAssembly();
            string[] resourceNames = asm.GetManifestResourceNames();
            for (int i = 0; i < resourceNames.Length; ++i)
            {
                if (string.Compare(resourceName, resourceNames[i], StringComparison.OrdinalIgnoreCase) == 0)
                {
                    using (StreamReader resourceReader = new StreamReader(asm.GetManifestResourceStream(resourceNames[i])))
                    {
                        if (resourceReader != null)
                        {
                            return resourceReader.ReadToEnd();
                        }
                    }
                }
            }
            return null;
        }

        internal class PictureDispMaker : AxHost
        {
            private PictureDispMaker() : base("") { }

            static public stdole.IPictureDisp ConvertImage(Image image)
            {
                return (stdole.IPictureDisp)GetIPictureDispFromPicture(image);
            }

            static public stdole.IPictureDisp ConvertIcon(Icon icon)
            {
                return ConvertImage(icon.ToBitmap());
            }
        }

        #endregion
    }
}
